/*
 * $Id: DNSCache.java 292 2006-02-28 23:30:47Z davidb $
 * 
 * Copyright (c) 2005 VeriSign. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer. 2. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution. 3. The name of the author may not
 * be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN
 * NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */

package se.rfc.unbound.cache;

import java.util.Properties;

import org.xbill.DNS.*;

import se.rfc.unbound.Request;
import se.rfc.unbound.SMessage;

/**
 * This is the interface for the main "name" cache--the cache that stores the
 * results of DNS queries. All entries in this cache are indexed by
 * <qname,qtype,qclass> or, in some cases, <qname,qclass>.
 * 
 * @author davidb
 */

public interface DNSCache
{

  /**
   * Initialize the cache module.
   * 
   * @param config
   * @throws Exception
   */
  public void init(Properties config) throws Exception;

  /**
   * Lookup a given question in the cache. The cache implementation is
   * responsible for forming the results into an entire response message.
   * 
   * The response message will be complete except for: header flags and ID and
   * EDNS0 status.
   * 
   * @param qname the name to look for, must not be null.
   * @param qtype the type to look for, must be a valid type (i.e., not
   *          Type.ANY).
   * @param qclass the class to look for, must be a valid class (usually
   *          DClass.IN).
   * 
   * @return a DNS response message if found in the cache, null if not.
   */
  public SMessage lookup(Name qname, int qtype, int qclass);

  /**
   * Stores the given response message in the cache. It is up to the cache
   * implementation to decide how to break the message down for caching, and
   * to handle credibility calculations and decisions.
   * 
   * @param response The response message to cache.
   * @param request The original request that generated this response.
   * @param isReferral If true, then the given message to be stored is a
   *          referral. The cache implementation may use this as a hint.
   */
  public void store(SMessage response, Request request, boolean isReferral);

  /**
   * Given a name and class, find the closest delegation point.
   * 
   * @param qname the name to look for, must not be null.
   * @param qclass the class to look for, must be a valid class.
   * 
   * @return a REFERRAL type message for the closest delegation point. Returns
   *         null when no delegation point could be found, indicating that the
   *         cache is unprimed for the given class.
   */
  public SMessage findDelegation(Name qname, int qclass);
  
  /**
   * Given a name and class, find and synthesize a CNAME from a cached parent
   * DNAME.
   * 
   * @param qname the name to look for, must not be null.
   * @param qtype the type of the original query.
   * @param qclass the class to look for, must be a valid class.
   * 
   * @return a CNAME type message for the parent DNAME, or null if no such
   *         parent DNAME exists (or cache does not support the operation).
   */
  public SMessage findDNAME(Name qname, int qtype, int qclass);
}